#####################################################################################
#
#          :::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]:::
#      ____                __                                                      
#     / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __   
#    / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // /   
#   /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/    
#        /___/ /___/                 /_/    /_/           /___/ /_/                
#
#
#####################################################################################
#
# anekdot 0.8
#
# Description:
#	Displays anekdots on channels with UAFS support.
#
# Authors:  mrBuG <mrbug@eggdrop.org.ru> 
#
# Special thanks to Nitrogen <nitrogen@zlodei.ru> and nemon
#
# Based on anekdot.tcl by Bart[mdv]
#
# UAFS script for eggdrop by CoolCold <coolcold at coolcold.org>
#				http://eggdrop.org.ru/scripts/uafs_beta10.zip
#
# Official support: irc.eggdrop.org.ru @ #eggdrop
# 
#####################################################################################
#
#        
#
# :
# 1.  (anekdot.tcl)   scripts.
# 2.   (andekdots.dat)   data(  ,  )
# 3.   eggdrop.conf:
#    source scripts/anekdot.tcl
#
# :
#     : 
# !anekdot(!) [  |] -    [   |  ]
# !ankmode <notice|channel|none> -   [notice | |    ]
# !ankrnd(!astart, !) <delay XX|none> -    [  | ]
#    :
# !anekdot(!) [  |] -    [   |  ]
#
#####################################################################################
#
# :
#
# 0.8 (14  2006):
#   -        . (   )
#   -       . (  UAFS)
#   -  ::anekdot::matchanekdot   ::anekdot::anekdot. (
#       anekdot(list)        1,5)
#   -       . (    ,    
#       .   !ankrnd delay XX)
#   -      . (    +b.
#          . () branix Jun 6 2006, 10:38 PM ;) )
#   -    use-fastdeq. (  )
#   -   "". (    :) )
#
# 0.7 fixed (4  2006):
#   -     !ankrnd delay XX. (     )
#   * Fixed by Nitrogen <nitrogen@zlodei.ru> (thx nemon)
#   -     . (    +usecolors)
#   * Fixed by Nitrogen <nitrogen@zlodei.ru>
#
# 0.7 (16  2005):
#   -  :
#       Tcl error [::anekdot::startanekdot]: can't read "anekdot(time,#moj_kanal)": no such element in array
#   -         +usecolors. (  )
#   -        . (
#         .+chan        
#          : ankmode=none, ankrnd=none, delay=0)
#
# 0.6b (23  2004):
#   -    :
#       !anekdot(!) [  |] -    [   |  ]
#   -      . (    )
#   -   UAFS.tcl (Universal Anti-Flood Script for eggdrop by CoolCold <coolcold at coolcold.org>).
#     (   ,  flood protect  )
#
# 0.5b (19  2004):
#   -           . (  )
#
# 0.4b (18  2004):
#   -    :
#       !ankmode <notice|channel|none> -   [notice | |    ]
#
#####################################################################################
#
# :
# 1.    ,   ,      
#   .    ,     .
# 2.         (================================)
#    (   )  (set use-fastdeq 0)   "Fastdeq Settings".
# :
#   use-fastdeq          ,
#            ,   :
#     PUTSERV #channel1, #channel2 :      2  
#          use-fastdeq  0   
#        .
#   use-fastdeq    2 ,   ircd    ...
#            ;).
#
#####################################################################################
# Last version of egglib_pub can be grabbed here:
# http://eggdrop.org.ru/scripts/egglib_pub.zip

if { ![info exists egglib(ver)] } {
	putlog "***********************************************"
	putlog "             egglib_pub NOT FOUND !"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"
	die
}

if { [expr {$egglib(ver) < 1.52}] } {
	putlog "***********************************************"
	putlog "    YOUR VERSION OF egglib_pub IS TOO OLD !"
	putlog "   Download last version of egglib_pub here:"
	putlog "  http://eggdrop.org.ru/scripts/egglib_pub.zip"
	putlog "***********************************************"
	putlog " version installed : $egglib(ver)"
	putlog " version required: 1.5.2"
	die
}

if { ![info exists ::uafs::ver] } {
	putlog "***********************************************"
	putlog "               UAFS NOT FOUND !"
	putlog "       Download last version of UAFS here:"
	putlog "  http://eggdrop.org.ru/scripts/uafs_beta10.zip"
	putlog "***********************************************"
	putloglev d * "\[anekdot\] flood protect  : off"
	set anekdot(uafs) 0
} else {
	if { $::uafs::numver < "000010" } {
		putlog "***********************************************"
		putlog "        YOUR VERSION OF UAFS IS TOO OLD !"
		putlog "       Download last version of UAFS here:"
		putlog "  http://eggdrop.org.ru/scripts/uafs_beta10.zip"
		putlog "***********************************************"
		putloglev d * "\[anekdot\] flood protect  : off"
		set anekdot(uafs) 0
	} else {
		putloglev d * "\[anekdot\] flood protect  : on"
		set anekdot(uafs) 1
	}
}

foreach chan [channels] {
	if { [info exists anekdot(time,$chan)] } { set temp_anekdot(time,$chan) $anekdot(time,$chan) } else { set temp_anekdot(time,$chan) 1 }
}

if { [info exists anekdot(queue)] } { set temp_anekdot(queue) $anekdot(queue) } else { set temp_anekdot(queue) "" }

if { [info exists anekdot(lengthborder)] } { set temp_anekdot(lengthborder) $anekdot(lengthborder) } else { set temp_anekdot(lengthborder) 0 }

if { [info exists anekdot(uafs)] } { set temp_anekdot(uafs) $anekdot(uafs) } else { set temp_anekdot(uafs) 0 }

foreach p [array names anekdot *] { catch { putloglev d * "\[anekdot\] unsetting anekdot($p)";unset anekdot($p) } }

namespace eval anekdot {}

### Begin Fastdeq Settings
# set use-fastdeq 0
### End Fastdeq Settings

### Begin User Defined Channel Settings
setudef flag usecolors
setudef int anekdot-delay
setudef str ank-mode
setudef str ank-random
### End User Defined Channel Settings

set anekdot(file) "data/anekdots.dat"
set anekdot(searchlist) ""

set anekdot(version) "0.8"
set anekdot(authors) "mrBuG <mrbug@eggdrop.org.ru>"

foreach chan [channels] { set anekdot(time,$chan) $temp_anekdot(time,$chan) }

set anekdot(uafs) $temp_anekdot(uafs)

### Begin UAFS Settings
if { $anekdot(uafs) } {
	set anekdot(ufh) [::uafs::registerhandle "3:40" $::uafs::masks(host) "" "" "0"]
	set anekdot(floodufh) [::uafs::registerhandle "2:120"]
	if { $anekdot(ufh) < 0 && $anekdot(anekdotufh) < 0 } {
		putloglev d * "\[uafs\] unable to register handler for anekdot.tcl"
		putloglev d * "\[anekdot\] flood protect  : off"
		set anekdot(uafs) 0
	}
}
### End UAFS Settings

### Begin Colors Settings
#    ( )
set anekdot(nmbcolorall) "\00310"
#    ( )
set anekdot(nmbcolornow) "\00304"
#    ( )
set anekdot(nmbcolorbrd) "\00310"
#    
set anekdot(ankcolor) "\00303"
#   
set anekdot(splcolorbrd) "\00304"
### End colors Settings

bind pub -|- ! ::anekdot::pubanekdot
bind pub -|- !anekdot ::anekdot::pubanekdot
bind pub m|m ! ::anekdot::startanekdot
bind pub m|m !astart ::anekdot::startanekdot
bind pub m|m !ankrnd ::anekdot::startanekdot
bind pub m|m !ankmode ::anekdot::pubmodeanekdot

bind msg -|- !anekdot ::anekdot::msganekdot
bind msg -|- ! ::anekdot::msganekdot

proc ::anekdot::settimer {} {
	global anekdot

	if { [lsearch [timers] *::anekdot::timerandomanekdot*] == -1 } {
		putloglev d * "\[anekdot\] timer not found. setting up"
		timer 1 ::anekdot::timerandomanekdot
		foreach chan [channels] {
			set anekdot(time,$chan) 1
			::anekdot::modeanekdot $chan
		}
	} else {
		putloglev d * "\[anekdot\] timer alredy set. not setting"
	}

}

proc ::anekdot::anekdot { matchmask } { 
	global anekdot

	if { $matchmask == "" } {
		set anekdotnumber [expr [rand $anekdot(counter)] +1]
	} else {

		if { [regexp "^\[0-9\]+$" $matchmask] } {
		set anekdotnumber $matchmask
			if { [expr $anekdotnumber > $anekdot(counter)] || [expr $anekdotnumber <= 0] } {
				return "err1"
			}
		} else {

			regsub -all {# (.*?)\}} [lsearch -inline -all $anekdot(searchlist) *$matchmask*] {} matchlistanekdot
			set matchlistanekdot [::egglib::strip_special $matchlistanekdot]
			if { $matchlistanekdot == "" } {
				return "err2"
			} else {
				putloglev d * "\[anekdot\] total matched  : [llength $matchlistanekdot]"
				set anekdotnumber [lindex $matchlistanekdot [rand [llength $matchlistanekdot]]]
			}

		}
	}

	regsub -all {\{(.*?)\ \#\ } [lsearch -inline -all $anekdot(searchlist) "$anekdotnumber\ #*"] {#} submatchlistanekdot
	regsub -all -- {\}\ } $submatchlistanekdot {} submatchlistanekdot; regsub -all -- {\}} $submatchlistanekdot {} submatchlistanekdot
	set submatchlistanekdot [lrange [split $submatchlistanekdot "\#"] 1 end]
	set listanekdot "\[$anekdotnumber/$anekdot(counter)\]"

	foreach flistanekdot $submatchlistanekdot {
		lappend listanekdot $flistanekdot
	}

	lappend listanekdot "================================"

	return $listanekdot

}

proc ::anekdot::loadanekdots {} { 
	global anekdot

	set anekdot(counter) 0
	set anekdot(searchlist) ""
	set tmp_anekdot_file [open $anekdot(file) r]
	putloglev d * "\[anekdot\] loading anekdots. please wait"
	while { [gets $tmp_anekdot_file line ] >=0 } {

		if { $line == "- NEW -" } { 
			incr anekdot(counter)
		}

		if { $line != "- NEW -" } {
			lappend anekdot(searchlist) "$anekdot(counter) # $line"
		}

	}
	close $tmp_anekdot_file 

	putloglev d * "\[anekdot\] load anekdots  : $anekdot(counter)"
} 

proc ::anekdot::msganekdot { nick uhost hand args } {
	::anekdot::outanekdot $nick $uhost $hand $nick [::egglib::strip_special $args] 0
}

proc ::anekdot::pubanekdot { nick uhost hand chan args } {
	if { ![::anekdot::activeanekdot $chan] } return
	::anekdot::outanekdot $nick $uhost $hand $chan [::egglib::strip_special $args] [::anekdot::noticeanekdot $chan]
}

proc ::anekdot::outanekdot { nick uhost hand chan text outputtype } {
	global anekdot lastbind botnick

	::egglib::log $nick $chan "anekdot" "$text"

	if { $anekdot(uafs) } {
		if { [::uafs::isflood $anekdot(ufh) $nick $uhost $chan] == 1 } {

			if { [::uafs::isflood $anekdot(floodufh) $nick $uhost $chan] == 1 } {
				if { [string equal $chan $nick] } {

					set mask [::egglib::tolower [::uafs::createmask $nick $uhost $::uafs::masks(host)]]
					newignore $mask $botnick "Flooding me with $lastbind" [expr round([::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]/60.0)]
					putserv "PRIVMSG $nick :You are flooder! Next time you may use \"$lastbind\" after [duration [expr round([::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]/60.0)*60]] (IGNORED)" -next
					putloglev d * "\[uafs\] flood detected : $mask on private"
					putloglev d * "\[uafs\] time ignore    : [duration [expr round([::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]/60.0)*60]]"
					return

				} else {

					if { [botisop $chan] || [botishalfop $chan] } {
						putserv "KICK $chan $nick :Stop flooding me with \"$lastbind\" bastard! Calm down and wait for [duration [::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]]" -next
						putloglev d * "\[uafs\] flood detected : $nick on $chan"
						putloglev d * "\[uafs\] $nick kicked"
						return
					} else {
						set mask [::egglib::tolower [::uafs::createmask $nick $uhost $::uafs::masks(host)]]
						newignore $mask $botnick "Flooding me with $lastbind" [expr round([::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]/60.0)]
						putserv "NOTICE $nick :You are flooder! Next time you may use \"$lastbind\" after [duration [expr round([::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]/60.0)*60]] (IGNORED)" -next
						putloglev d * "\[uafs\] flood detected : $mask on $chan"
						putloglev d * "\[uafs\] time ignore    : [duration [expr round([::uafs::peacetime $anekdot(floodufh) $nick $uhost $chan]/60.0)*60]]"
						return
					}

				}
			}

			if { [string equal $chan $nick] } {
				putserv "PRIVMSG $nick :You are flooder! Next time you may use \"$lastbind\" after [duration [::uafs::peacetime $anekdot(ufh) $nick $uhost $chan]]" -next
				putloglev d * "\[uafs\] flood detected : $nick on private"
				return
			} else {
				putserv "NOTICE $nick :You are flooder! Next time you may use \"$lastbind\" after [duration [::uafs::peacetime $anekdot(ufh) $nick $uhost $chan]]" -next
				putloglev d * "\[uafs\] flood detected : $nick on $chan"
				return
			}
		}
	}

	set listanekdot [::anekdot::anekdot $text]

	if { $listanekdot == "err1"} {
		if { $outputtype } {
			::egglib::outn $nick "     ."
			return
		} else {
			::egglib::out $nick $chan "     ."
			return
		}
	}

	if { $listanekdot == "err2"} {
		if { $outputtype } {
			::egglib::outn $nick ",     ."
			return
		} else {
			::egglib::out $nick $chan ",     ."
			return
		}
	}

	::anekdot::outputanekdot $chan $nick $listanekdot $outputtype

}

proc ::anekdot::startanekdot { nick uhost hand chan text } {
	global anekdot

	if { ![::anekdot::activeanekdot $chan] } { return }

	::egglib::publog $nick $chan "startanekdot" "$text"

	set arg [lindex $text 0]
	set argdelay [lindex $text 1]

	if { $arg == "" } {
		::egglib::outhc $nick $chan "!ankrnd" "\<delay XX\|none>"
		::egglib::outc $nick $chan " :\0035 [channel get $chan ank-random] \00306\[[channel get $chan anekdot-delay]\]\003."
		return
	}

	if { ![string match $arg "delay"] &&
		 ![string match $arg "none"] } {
		::egglib::outc $nick $chan " .  \0034!ankrnd\0035 <delay XX|none>\003"
		return
	}

	if { [string match $arg "delay"] && ( $argdelay == "" || $argdelay <= 0 || ![regexp "^\[0-9\]+$" $argdelay] ) } {
		::egglib::outc $nick $chan " .  \0034!ankrnd\0035 delay XX\003 ( \00306XX\003 - - )"
		return
	}

	if { [string match $arg "none"] } {
		if { ![::anekdot::randomanekdot $chan] } { return }
		channel set $chan anekdot-delay 0
		channel set $chan ank-random none
		savechannels
		set $anekdot(time,$chan) 1
		::egglib::outc $nick $chan "\00302    : \00312none\003."
		return
	}

	channel set $chan anekdot-delay $argdelay
	channel set $chan ank-random random
	savechannels
	set $anekdot(time,$chan) -1
	::egglib::outc $nick $chan "\00302    : \00312random\00306 \[[channel get $chan anekdot-delay]\]\003."
	set listanekdot [::anekdot::anekdot ""]

::anekdot::outputanekdot $chan $nick $listanekdot 0

}

proc ::anekdot::pubmodeanekdot { nick uhost hand chan text } {
	global anekdot

	::egglib::publog $nick $chan "anekdotmode" "$text"

	if { ![::anekdot::modeanekdot $chan] } {
		::egglib::outc $nick $chan "         ."
		return
	}

	set arg [::egglib::tolower [lindex $text 0]]

	if { [llength $text] == 0 } {
		::egglib::outhc $nick $chan "!ankmode" "<notice|channel|none>"
		set mode [channel get $chan ank-mode]
			::egglib::outc $nick $chan " :\0035 $mode\003."
			return
	}

	if { ![string match $arg "notice"] &&
		 ![string match $arg "channel"] &&
		 ![string match $arg "none"] } {
		::egglib::outc $nick $chan " .  \0034!ankmode\0035 <notice|channel|none>\003"
		return
	}

	channel set $chan ank-mode $arg
	savechannels
	
	::egglib::outc $nick $chan "\0032    : \00312$arg\003."

}

proc ::anekdot::timerandomanekdot {} {
	global anekdot

	foreach chan [channels] {

		if { ![info exist anekdot(time,$chan)] } {
		 	set anekdot(time,$chan) 1
		}

		::anekdot::modeanekdot $chan

		if { [::anekdot::activeanekdot $chan] && [::anekdot::randomanekdot $chan] } {
			set intervalanekdot($chan) [channel get $chan anekdot-delay]
			if { $intervalanekdot($chan) == 0 || ![regexp "^\[0-9\]+$" $intervalanekdot($chan)] || ![botonchan $chan] || [::anekdot::botonchannel $chan] } { continue }
			if { $intervalanekdot($chan) <= $anekdot(time,$chan) } { 
				set listanekdot [::anekdot::anekdot ""]
				::anekdot::outputanekdot $chan "" $listanekdot 0
				set anekdot(time,$chan) 0
			}
			incr anekdot(time,$chan)
		}
	}
	timer 1 ::anekdot::timerandomanekdot
}


proc ::anekdot::activeanekdot { chan } {
	set mode [channel get $chan ank-mode]
	if { ![string match $mode "none"] } {
		if { [string match $mode "notice"] || [string match $mode "channel"] } { return 1}
	}
	return 0
}

proc ::anekdot::noticeanekdot { chan } {
	set mode [channel get $chan ank-mode]
	if { [string match $mode "notice"] } { return 1 }
	return 0
}

proc ::anekdot::randomanekdot { chan } {
	set mode [channel get $chan ank-random]
	if { [string match $mode "random"] } { return 1 }
	return 0
}

proc ::anekdot::modeanekdot { chan } {
	set outreturn 1

	set mode [channel get $chan ank-mode]
	if { ![string match $mode "none"] &&
		![string match $mode "notice"] &&
		![string match $mode "channel"] } {
			channel set $chan ank-mode none
			savechannels
			set outreturn 0
	}

	set mode [channel get $chan ank-random]
	if { ![string match $mode "none"] &&
		![string match $mode "random"] } {
			channel set $chan ank-random none
			savechannels
			set outreturn 0
	}

	return $outreturn
}

proc ::anekdot::botonchannel { chan } {
	if { [llength [chanlist $chan]] <= 1 } {
		return 1
	} elseif { [expr [llength [chanlist $chan]] - 1] == [llength [chanlist $chan b]] } {
		return 1
	} else {
	return 0
	}
}

proc ::anekdot::outputanekdot { chan nick listanekdot outputtype } {
	global anekdot

	foreach anpub $listanekdot {
		if { $outputtype } {
			putserv "NOTICE $nick :$anpub"
		} elseif { [string equal $chan $nick] } {
			putserv "PRIVMSG $nick :$anpub"
		} else {
			if { [channel get $chan usecolors] } {
				regsub -all -- {\[} $anpub "$anekdot(nmbcolorbrd)\[$anekdot(nmbcolornow)" anpub
				regsub -all -- {\/} $anpub "$anekdot(nmbcolorall)\/" anpub
				regsub -all -- {\]} $anpub "$anekdot(nmbcolorbrd)\]\00300" anpub
				regsub -all -- {================================} $anpub "$anekdot(splcolorbrd)================================\00300" anpub
				putserv "PRIVMSG $chan :$anekdot(ankcolor)$anpub\00300"
			} else {
				putserv "PRIVMSG $chan :$anpub"
			}
		}
	}

}

::anekdot::loadanekdots

::anekdot::settimer

::egglib::srcloadlog "anekdot" $anekdot(version) $anekdot(authors)
